//
// Patron pour la generation d'une classe derivee dans ROOT
// FGuinevereDataReader derive de FBlockDataReader
//

#ifndef FGuinevereDataReader_h
#define FGuinevereDataReader_h

#include "FBlockDataReader.h"
#include "FParamBrut.h"

// Taille maximale des fichiers de donnees fournis par le système Gandalf
//const Int_t maxGuinevereDataFileSize = 81920; // ONLY FOR MY PERSONNAL TESTS IN JULY 2011 (10 buffers of 8192 bytes)
const Int_t maxGuinevereDataFileSize = 2130706432; // L'ancienne limite trouvee avec un ll sur le fichier d'exemple
	  				           // "Gandalf-20110225-134930.dat" etait de 2131206144 ...
					           // 
					           // NB: cette valeur a ete confirmee le 22/07/2011 par G.Dargaud.
					           //
//
// Definition des types de donnees GUINEVERE pour RHB
// 
const Int_t ITC = 15;              // Initial Time Counter
const Int_t AKI_TM = 14;           // Acquisition Dead Time
const Int_t REACTOR = 6;           // VENUS Reactor
const Int_t TARGET_TEMP = 1;       // Target Accelerator Temperature
const Int_t SILI = 2;              // Silicium Monitors 1 to 3 and Coinc Neutron Monitor
const Int_t ALPHA_GE = 3;          // Alpha Monitors 1&2 and Pulse GENEPI
const Int_t FOUR_GRP = 4;          // Mix of Fission Intensity Chambers and Analog Channels 1&2
const Int_t A3toA6 = 5;            // Analog Channels 3 to 6
const Int_t CEA_FIS = 8;           // CEA Fission Chambers 1 to 4
const Int_t SCKFIS_LOG1toLOG3 = 9; // SCK Fission Chamber and Logical Channels 1 to 3
const Int_t LOG4toLOG7 = 10;       // Logical Channels 4 to 7
const Int_t LOG8toLOG11 = 11;      // Logical Channels 8 to 11
const Int_t LOG12 = 12;            // Logical Channel 12
const Int_t FAISC = 13;            // Faisceau

// 
// Declaration des formats de donnees GUINEVERE par champs de bits (ils sont tous sur 64 bits)
//
// Rappel: pour une lecture sur machine X86 (Little Endian), le 1er champ commence au bit0
//
// Format general d'une donnee GUINEVERE
typedef struct Guinevere_data{
  unsigned long long data : 57; // Data value
  unsigned data_bit57 : 1;      // [57]
  unsigned data_number : 2;     // [59-58]
  unsigned data_type : 4;       // [63..60]
} Guinevere_data;

// Format d'une donnee Acquisition
typedef struct Acqui_data{
  unsigned long long acqui_measure : 42; // ITC or AKI_TM
  unsigned acqui_unused : 16;
  unsigned acqui_number : 2;             // [59-58]
  unsigned acqui_type : 4;               // [63..60]
} Acqui_data;

//////////////////////////////////////////////////
// Formats des donnees issues du Reacteur Venus //
//////////////////////////////////////////////////
//
// Format de donnees special Safety Reacteur Venus
typedef struct Safety_Rod_Reactor_data{
  unsigned safety_rod_position : 8; // Safety Rod position
  unsigned safety_rod_unused1 : 24; 
  unsigned safety_rod_time : 16;    // Saftey Rod time
  unsigned safety_rod_unused2 : 9; 
  unsigned safety_rod_bit57 : 1;    // [57]
  unsigned safety_rod_number : 2;   // [59-58]
  unsigned safety_rod_type : 4;     // [63..60]
} Safety_Rod_Reactor_data;
//
// Formats des donnees issues du Reacteur Venus
// 1er type: bits 48 a 56 inutilises
typedef struct Reactor1_data{
  unsigned reactor1_measure : 32; // Temperature_1/Temperature_2 or Start_A/Start_B
  unsigned reactor1_time : 16;    // Time
  unsigned reactor1_unused : 9;
  unsigned reactor1_bit57 : 1;    // [57]
  unsigned reactor1_number : 2;   // [59-58]
  unsigned reactor1_type : 4;     // [63..60]
} Reactor1_data;
//
// 2eme type: bits 48 a 55 inutilises
typedef struct Reactor2_data{
  unsigned reactor2_measure : 32; // Temp_Reac_i or Ion_i
  unsigned reactor2_time : 16;    // Time
  unsigned reactor2_unused : 8;
  unsigned reactor2_channel : 2;  // [57-56]
  unsigned reactor2_number : 2;   // [59-58]
  unsigned reactor2_type : 4;     // [63..60]
} Reactor2_data;
//
// 3eme type: bits 48 a 54 inutilises
typedef struct Reactor3_data{
  unsigned reactor3_control_rod : 32; // Control_Rod_i
  unsigned reactor3_time : 16;        // Time
  unsigned reactor3_unused : 7;
  unsigned reactor3_channel : 2;      // [56-55]
  unsigned reactor3_bit57 : 1;        // [57]
  unsigned reactor3_number : 2;       // [59-58]
  unsigned reactor3_type : 4;         // [63..60]
} Reactor3_data;
//
///////////// Fin donnees Reactor Venus //////////

// Format d'une donnee de type Reactor Count
typedef struct ReactorCount_data{
  unsigned reactorCount_starta : 17;  // Start_A
  unsigned reactorCount_unused1 : 15;
  unsigned reactorCount_startb : 17;  // Start_B
  unsigned reactorCount_unused2 : 9;
  unsigned reactorCount_number : 2;   // [59-58]
  unsigned reactorCount_type : 4;     // [63..60]
} ReactorCount_data;

// Format des donnees Moniteurs SiLi ou Alpha
typedef struct Monitor_data{
  unsigned long long monitor_time : 42; // Mi_Ti
  signed monitor_val : 12;              // Vali
  unsigned monitor_unused : 4;
  unsigned monitor_number : 2;          // [59-58]
  unsigned monitor_type : 4;            // [63..60]
} Monitor_data;

// Format des donnees Moniteurs SiLi Coinc ou Count Pulse Genepi
typedef struct CountCoinc_data{
  unsigned long long countCoinc_time : 42; // T6 or Tge
  unsigned countCoinc_nbCoinc : 15;        // Nb Coinc or Nb Count Ge
  unsigned countCoinc_bit57 : 1;           // [57]
  unsigned countCoinc_number : 2;          // [59-58]
  unsigned countCoinc_type : 4;            // [63..60]
} CountCoinc_data;

// Format d'une donnnee Val Genepi
typedef struct ValGenepi_data{
  unsigned long long valGenepi_time : 42; // Tge
  signed valGenepi_val : 12;              // Val_Ge
  unsigned valGenepi_unused : 3;
  unsigned valGenepi_bit57 : 1;           // [57]
  unsigned valGenepi_number : 2;          // [59-58]
  unsigned valGenepi_type : 4;            // [63..60]
} ValGenepi_data;

// Format des donnees Analogiques & Courant des chambres de fission
// Val
typedef struct ValAnaFis_data{
  unsigned long long valAnaFis_time : 42; // Time Ci or Ai
  signed valAnaFis_val : 12;              // Val Ci or Ai
  unsigned valAnaFis_unused : 3;
  unsigned valAnaFis_number : 3;          // [59..57]
  unsigned valAnaFis_type : 4;            // [63..60]
} ValAnaFis_data;
// Nb_Count
typedef struct CountAnaFis_data{
  unsigned long long countAnaFis_time : 42; // Time Ci or Ai
  unsigned countAnaFis_nbCount : 15;        // Nb_Count Ci or Ai
  unsigned countAnaFis_number : 3;          // [59..57]
  unsigned countAnaFis_type : 4;            // [63..60]
} CountAnaFis_data;

// Format des donnes Digital
typedef struct Digital_data{
  unsigned long long digital_time : 42; // Time Di
  unsigned digital_nbCount : 15;        // Nb_Count Di
  unsigned digital_bit57 : 1;           // [57]
  unsigned digital_number : 2;          // [59-58]
  unsigned digital_type : 4;            // [63..60]
} Digital_data;

// Format des donnees Faisceau
// FAISC1
typedef struct Faisc1_data{
  unsigned long long faisc1_time : 42;
  unsigned faisc1_unused1 : 2;
  unsigned faisc1_uPulse : 13; 
  unsigned faisc1_unused2 : 1;   // [57]
  unsigned faisc1_number : 2;    // [59-58]
  unsigned faisc1_type : 4;      // [63..60]
} Faisc1_data;
// FAISC2
typedef struct Faisc2_data{
  unsigned faisc2_aIntegral : 19;
  unsigned faisc2_unused : 1;
  unsigned faisc2_tPulse : 14; 
  unsigned faisc2_tCycle_tBeam : 24;
  unsigned faisc2_number : 2;  // [59-58]
  unsigned faisc2_type : 4;    // [63..60]
} Faisc2_data;
// FAISC3
typedef struct Faisc3_data{
  unsigned long long faisc3_sIntegral : 55;
  unsigned faisc3_unused : 3;
  unsigned faisc3_number : 2;  // [59-58]
  unsigned faisc3_type : 4;    // [63..60]
} Faisc3_data;

class FGuinevereDataReader : public FBlockDataReader {

  protected:
    FDevice *GuinevereDevice;
    THashList labelToName; // liste intermediaire pour accelerer l'acces aux parametres
    Int_t cptParam;

  private:
    void AddAcquiParams(void);
    void AddReactorParams(void);
    void AddTargetParams(void);
    void AddSiliMonitorParams(void);
    void AddAlphaGeMonitorParams(void);
    void AddFissionAndA1A2Params(void);
    void AddA3toA6Params(void);
    void AddCEAFisParams(void);
    void AddSckFisAndLOG1toLOG3Params(void);
    void AddLOG4toLOG7Params(void);
    void AddLOG8toLOG11Params(void);
    void AddLOG12Params(void);
    void AddFaiscParams(void);
    virtual Int_t GetNextEvent(void);
    void ReadData(void);
    void ReadAcquiParams(void);
    void ReadReactorParams(void);
    void ReadTargetTempParams(void);
    void ReadSiLiParams(void);
    void ReadAlphaGeParams(void);
    void ReadAnaAndFisParams(void);
    void ReadAna3ToAna6Params(void);
    void SetValCi(Int_t number);
    void SetValAi(Int_t type, Int_t number);
    void SetCountCi(Int_t number);
    void SetCountAi(Int_t type, Int_t number);
    void ReadDigitalChannels(void);
    void ReadFaiscParams(void);
    Int_t GetNewBuffer(void);
    void ChangeFileName(void);

  public:
    FGuinevereDataReader(void);    // constructeur par defaut
    FGuinevereDataReader(const FGuinevereDataReader &ap); // constructeur par copie

    ~FGuinevereDataReader(void);		// desctructeur;

#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
    virtual void Copy(TObject & obj) const;
#else
    virtual void Copy(TObject & obj);
#endif
    FGuinevereDataReader& operator = (const FGuinevereDataReader &hp); // operateur =

    virtual void InitFromEnv(TEnv *env);
    virtual void SetDevice(FDevice *dev=0);
    virtual void InitListOfParamsBruts(void);
    virtual void SetParamBrutsValues(void);
    virtual FParamBrut *GetParamBrut(Int_t labelRHB);
    virtual void AddParam(FParamBrut *p=0); // Ajout d'un parametre aux differentes listes

    ClassDef(FGuinevereDataReader,1)

};

#endif

