//
// Patron pour la generation d'une classe derivee dans ROOT
// FH2D derive de TH1F
//

#ifndef FH2D_h
#define FH2D_h

#include "TH2D.h"
#include "FH2Att.h"

class FH2D : public TH2D, public FH2Att {

  protected:
    virtual void Init(void);       // initialisation

  public:
    FH2D(void);    // constructeur par defaut
    FH2D(const char* name, const char* title, Int_t nbinsx, Axis_t xlow, Axis_t xup, Int_t nbinsy, Axis_t ylow, Axis_t yup);
    FH2D(const char* name, const char* title, Int_t nbinsx, const Double_t* xbins, Int_t nbinsy, Axis_t ylow, Axis_t yup);
    FH2D(const char* name, const char* title, Int_t nbinsx, Axis_t xlow, Axis_t xup, Int_t nbinsy, const Double_t* ybins);
    FH2D(const char* name, const char* title, Int_t nbinsx, const Double_t* xbins, Int_t nbinsy, const Double_t* ybins);
    FH2D(const char* name, const char* title, Int_t nbinsx, const Float_t* xbins, Int_t nbinsy, const Float_t* ybins);
    FH2D(const FH2D &ap); // constructeur par copie

    ~FH2D(void);		// desctructeur;

#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
    virtual void Copy(TObject & obj) const;
#else
    virtual void Copy(TObject & obj);
#endif
    FH2D& operator = (const FH2D &hp); // operateur =

    virtual TH1 *GetHisto(void);
    virtual void SavePrimitive(ostream &out, Option_t *opt="");

    virtual void Paint(Option_t *option="");
    virtual void ToggleActivate(Int_t val=1) {FHistAtt::ToggleActivate(val);} // *TOGGLE* *GETTER=IsActivated
    virtual void ToggleCanRebin(Int_t val=1) {FHistAtt::ToggleCanRebin(val);} // *TOGGLE* *GETTER=CanRebin
    
    ClassDef(FH2D,1)

};

#endif

