// 
// Patron pour la generation d'une classe derivee dans ROOT
// MFPCManager derive de TGCompositeFrame
// 
 
#ifndef MFPCManager_h 
#define MFPCManager_h 
 
#include "TROOT.h"
#include "TGClient.h"
#include "FEventProcessorThread.h" 
#include "FParamCalc.h"
#include "MCBLabel.h" 
#include "MTELabel.h" 
 
class MFPCManager : public TGCompositeFrame { 

	enum ElementId 
	{
	 CB_loPC,
	 TE_Expression,
	 B_AddPC,
	 B_RemovePC,
	 B_UpdatePCPar,
	 B_ReadPCPar,
	 CB_loPCFormula,
	 TGN_Par    // Il faut que se soit le dernier de la liste
	};
 
   protected: 
	FEventProcessorThread 	*fept;
	TList			*listOfParCal;
	MCBLabel		*loPC;
	MTELabel		*enterExpression;
	TGCompositeFrame	*tParCalFrame;
	TGCompositeFrame	*readUpdateFrame;
	FParamCalc		*currentParCal;
	TGTextButton		*update,*read;
	MCBLabel		*loPCFormula;

   protected: 
        virtual void Init(void);       // initialisation 
 
   public:	 
	MFPCManager(const TGWindow *p=0,UInt_t w=500,UInt_t h=300);
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,12,0)
	MFPCManager(const MFPCManager &ap); // constructeur par copie 
#endif
	
	~MFPCManager(void);		// desctructeur; 
	 
#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
   	virtual void Copy(TObject & obj) const;
#else
   	virtual void Copy(TObject & obj);
#endif
	MFPCManager& operator = (const MFPCManager &hp); // operateur = 
 
	virtual Bool_t ProcessMessage(Long_t msg, Long_t par1, Long_t par2); 

	virtual FEventProcessorThread *GetFEventProcessorThread(void)
		{return fept;}
	virtual void SetFEventProcessorThread(FEventProcessorThread *f=0)
		{fept=f;}

	virtual void AddNewParCal(void);
	virtual FParamCalc *AddParCal(void);
	virtual void RemoveParCal(void);
	virtual void BuildListeParCal(void);
	virtual void SetListeParCal(void);
 	virtual TList *GetListeOfParCal(void);
	virtual void SetParamCalcExpression(void);
	virtual void GetParCalParamValues(void);
	virtual void SetParCalParamValues(void);
	virtual void NewParCalc(char* s) {Emit("NewParCalc(char*)",s);} // *SIGNAL*
	virtual void RemParCalc(char* s) {Emit("RemParCalc(char*)",s);} // *SIGNAL*
	virtual void NewParameter(void);
	virtual void NewOr(void);
	virtual void NewTextParameter(void);
	virtual void AddTextParam(char *s);
	virtual void RemoveTextParam(char *s);
	virtual Bool_t CheckValueChanged(Int_t i);
	virtual void Reset(void);
	virtual void WarningBox(char const *title="Warning",char const *msg="Warning");

    ClassDef(MFPCManager,1)
 
}; 
 
#endif 
 
