// 
// Patron pour la generation d'une classe derivee dans ROOT
// RHVEditor derive de TGCompositeFrame
// 
 
#ifndef RHVEditor_h 
#define RHVEditor_h 
 
#include "TROOT.h"
#include "TGFrame.h"
#include "TCanvas.h"
#include "TGButton.h"
#include "MCBListManager.h"
#include "FPageLayout.h"
#include "FBooklet.h"
 
class RHVEditor : public TGCompositeFrame { 

   protected: 
	TList	*listOfHistos;
	TList	*listOfLayouts;
	TList	*listOfBooklets;
	TCanvas	*canvasEditor;
   	MCBListManager  *lmLo;
  	MCBListManager  *lmBook;
  	MCBListManager  *lmPage;
	
	TGCompositeFrame *cfOscBout;
	
	TGLayoutHints	*eX;
	TGLayoutHints	*eXeY;
  	
	FBooklet *editingBooklet;

   protected: 
        virtual void Init(void);       // initialisation 
 
   public:	 
	RHVEditor(const TGWindow *p=0,UInt_t w=500,UInt_t h=300);
#if ROOT_VERSION_CODE <= ROOT_VERSION(5,12,0)
	RHVEditor(const RHVEditor &ap); // constructeur par copie 
#endif
	
	~RHVEditor(void);		// desctructeur; 
	 
#if ROOT_VERSION_CODE >= ROOT_VERSION(3,4,0)
   	virtual void Copy(TObject & obj) const;
#else
   	virtual void Copy(TObject & obj);
#endif
	RHVEditor& operator = (const RHVEditor &hp); // operateur = 
 
	virtual Bool_t ProcessMessage(Long_t msg, Long_t par1, Long_t par2); 

	virtual TCanvas *GetCanvas(void) {return canvasEditor;}
	virtual void SetListOfHistos(TList *l){listOfHistos=l;}
	virtual void SetListOfLayouts(TList *l)
	  {listOfLayouts=l;lmLo->SetList(l);lmLo->RebuildList();}
	virtual void SetListOfBooklets(TList *l)
	  {listOfBooklets=l;lmBook->SetList(l);lmBook->RebuildList();}
	virtual void Reset(void);
	virtual void WarningBox(char const *title="Warning",char const *msg="Warning");
  	virtual void ClearCanvasEditor(void);
  	virtual void ClearCurrentPad(void);
	virtual void UpdateListOfLayouts(void);
  	virtual void DisplayPageLayout(char *s);
	virtual void UpdateListOfBooklets(void);
  	virtual void SetNewPageLayout(Char_t *loName);
  	virtual void RemovePageLayout(Char_t *plName);
  	virtual void ModifyPageLayout(Char_t *plName);
  	virtual Bool_t IsPageLayoutUsed(FPageLayout *pl);
  	virtual void SetNewBooklet(Char_t *blName);
  	virtual void SelectBooklet(char *s);
  	virtual void RemoveBooklet(Char_t *bName);
  	virtual void SetNewPage(Char_t *pName);
  	virtual void SelectPage(char *s);
  	virtual void RemovePage(Char_t *pName);
  	virtual void ModifyPage(Char_t *pName);
	
	virtual void PageAdded(char *bookletName, char *pageName) 
	             {Long_t args[2];
		      args[0]=(Long_t)bookletName;
		      args[1]=(Long_t)pageName;
		      #if ROOT_VERSION_CODE < ROOT_VERSION(6,0,0)
		      Emit("PageAdded(char *s,char *s2)",args);
		      #else
		      Emit("PageAdded(char *,char *)",args);}
		      #endif
	virtual void PageModified(char *bookletName, char *pageName) 
	             {Long_t args[2];
		      args[0]=(Long_t)bookletName;
		      args[1]=(Long_t)pageName;
		      Emit("PageModified(char *s,char* s2)",args);}
	virtual void PageRemoved(char *bookletName, char *pageName) 
	             {Long_t args[2];
		      args[0]=(Long_t)bookletName;
		      args[1]=(Long_t)pageName;
		      #if ROOT_VERSION_CODE < ROOT_VERSION(6,0,0)
		      Emit("PageRemoved(char *s,char *s2)",args);
		      #else
		      Emit("PageRemoved(char *,char *)",args);}
		      #endif

	virtual void PageLayoutAdded(char *s) 
	             {Emit("PageLayoutAdded(char *s,)",s);}
	virtual void PageLayoutModified(char *s) 
	             {Emit("PageLayoutModified(char *s)",s);}
	virtual void PageLayoutRemoved(char *s) 
	             {Emit("PageLayoutRemoved(char *s)",s);}

	virtual void BookletAdded(char *s)
	             {
		      #if ROOT_VERSION_CODE < ROOT_VERSION(6,0,0)
		      Emit("BookletAdded(char *s)",s);
		      #else
		      Emit("BookletAdded(char *)",s);}
		      #endif
	virtual void BookletModified(char *s) 
	             {Emit("BookletModified(char *s)",s);}
	virtual void BookletRemoved(char *s) 
	             {
		      #if ROOT_VERSION_CODE < ROOT_VERSION(6,0,0)
		      Emit("BookletRemoved(char *s)",s);
		      #else
		      Emit("BookletRemoved(char *)",s);}
		      #endif

    ClassDef(RHVEditor,1)
 
}; 
 
#endif 
 
